@@ -3,14 +3,13 @@ package Crypt::AuthEnc::EAX;
 use strict;
 use warnings;
 
-use Exporter 'import';
+use base qw(Crypt::AuthEnc Exporter);
 our %EXPORT_TAGS = ( all => [qw( eax_encrypt_authenticate eax_decrypt_verify )] );
 our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 our @EXPORT = qw();
 
 use CryptX;
 use Crypt::Cipher;
-use base 'Crypt::AuthEnc';
 
 ### the following methods/functions are implemented in XS:
 # - _new
