@@ -3,16 +3,16 @@ package Crypt::PK::DSA;
 use strict;
 use warnings;
 
-use Exporter 'import';
+require Exporter; our @ISA = qw(Exporter); ### use Exporter 'import';
 our %EXPORT_TAGS = ( all => [qw( dsa_encrypt dsa_decrypt dsa_sign_message dsa_verify_message dsa_sign_hash dsa_verify_hash )] );
 our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 our @EXPORT = qw();
 
-use CryptX;
-use Crypt::PK;
-use Crypt::Digest 'digest_data';
 use Carp;
-use MIME::Base64 qw(encode_base64 decode_base64);
+use CryptX qw(_encode_json _decode_json);
+use Crypt::Digest 'digest_data';
+use Crypt::Misc qw(read_rawfile encode_b64u decode_b64u encode_b64 decode_b64 pem_to_der der_to_pem);
+use Crypt::PK;
 
 sub new {
   my ($class, $f, $p) = @_;
@@ -25,9 +25,9 @@ sub export_key_pem {
   my ($self, $type, $password, $cipher) = @_;
   my $key = $self->export_key_der($type||'');
   return unless $key;
-  return Crypt::PK::_asn1_to_pem($key, "DSA PRIVATE KEY", $password, $cipher) if $type eq 'private';
-  return Crypt::PK::_asn1_to_pem($key, "DSA PUBLIC KEY") if $type eq 'public';
-  return Crypt::PK::_asn1_to_pem($key, "PUBLIC KEY") if $type eq 'public_x509';
+  return der_to_pem($key, "DSA PRIVATE KEY", $password, $cipher) if $type eq 'private';
+  return der_to_pem($key, "DSA PUBLIC KEY") if $type eq 'public';
+  return der_to_pem($key, "PUBLIC KEY") if $type eq 'public_x509';
 }
 
 sub import_key {
@@ -47,7 +47,7 @@ sub import_key {
     $data = $$key;
   }
   elsif (-f $key) {
-    $data = Crypt::PK::_slurp_file($key);
+    $data = read_rawfile($key);
   }
   else {
     croak "FATAL: non-existing file '$key'";
@@ -55,18 +55,18 @@ sub import_key {
   croak "FATAL: invalid key data" unless $data;
 
   if ($data =~ /-----BEGIN (DSA PRIVATE|DSA PUBLIC|PRIVATE|PUBLIC) KEY-----(.*?)-----END/sg) {
-    $data = Crypt::PK::_pem_to_binary($data, $password);
+    $data = pem_to_der($data, $password);
     return $self->_import($data);
   }
   elsif ($data =~ /---- BEGIN SSH2 PUBLIC KEY ----(.*?)---- END SSH2 PUBLIC KEY ----/sg) {
-    $data = Crypt::PK::_pem_to_binary($data);
+    $data = pem_to_der($data);
     my ($typ, $p, $q, $g, $y) = Crypt::PK::_ssh_parse($data);
-    return $self->_import_hex($p, $q, $g, undef, $y) if $typ && $p && $q && $g && $y && $typ eq 'ssh-dss';
+    return $self->_import_hex(unpack('H*',$p), unpack('H*',$q), unpack('H*',$g), undef, unpack('H*',$y)) if $typ && $p && $q && $g && $y && $typ eq 'ssh-dss';
   }
   elsif ($data =~ /ssh-dss\s+(\S+)/) {
-    $data = decode_base64($1);
+    $data = decode_b64("$1");
     my ($typ, $p, $q, $g, $y) = Crypt::PK::_ssh_parse($data);
-    return $self->_import_hex($p, $q, $g, undef, $y) if $typ && $p && $q && $g && $y && $typ eq 'ssh-dss';
+    return $self->_import_hex(unpack('H*',$p), unpack('H*',$q), unpack('H*',$g), undef, unpack('H*',$y)) if $typ && $p && $q && $g && $y && $typ eq 'ssh-dss';
   }
   else {
     return $self->_import($data);
